@@ -84,7 +84,7 @@ extern "C" {
 
         try {
             factory = new PrimeFactory (String ("ja_JP"),
-                                        String ("53ad00ca-0eb7-4495-bec4-7cdb52f367d7"),
+                                        String ("ff7ea8c6-2fd5-494c-a691-e6bd0101b7d3"),
                                         _scim_config);
         } catch (...) {
             delete factory;
@@ -141,8 +141,8 @@ WideString
 PrimeFactory::get_authors () const
 {
     return utf8_mbstowcs (
-        _("Copyright (C) 2005 Hiroyuki Ikezoe <poincare@ikezoe.net>\n"
-          "Copyright (C) 2005 Takuro Ashie <ashie@homa.ne.jp>"));
+        _("Copyright (C) 2005 Takuro Ashie <ashie@homa.ne.jp>\n"
+          "Copyright (C) 2005 Hiroyuki Ikezoe <poincare@ikezoe.net>"));
 }
 
 WideString
